@@ -3,9 +3,6 @@ Linkedin OAuth support
 
 No extra configurations are needed to make this work.
 """
-import logging
-logger = logging.getLogger(__name__)
-
 from xml.etree import ElementTree
 from xml.parsers.expat import ExpatError
 
@@ -53,7 +50,7 @@ class LinkedinAuth(ConsumerBasedOAuth):
     SETTINGS_KEY_NAME = 'LINKEDIN_CONSUMER_KEY'
     SETTINGS_SECRET_NAME = 'LINKEDIN_CONSUMER_SECRET'
 
-    def user_data(self, access_token):
+    def user_data(self, access_token, *args, **kwargs):
         """Return user data provided"""
         fields_selectors = LINKEDIN_FIELD_SELECTORS + \
                            setting('LINKEDIN_EXTRA_FIELD_SELECTORS', [])
@@ -65,10 +62,6 @@ class LinkedinAuth(ConsumerBasedOAuth):
         except (ExpatError, KeyError, IndexError):
             return None
 
-    @classmethod
-    def enabled(cls):
-        return True
-
 
 def to_dict(xml):
     """Convert XML structure to dict recursively, repeated keys entries
